@@ -320,11 +320,12 @@ class FloatWithUnit(float):
         super(FloatWithUnit, self).__init__(val)
 
     def __repr__(self):
-        s = super(FloatWithUnit, self).__repr__()
-        return "{} {}".format(s, self._unit)
+        return super(FloatWithUnit, self).__repr__()
+        #return "{} {}".format(s, self._unit)
 
     def __str__(self):
-        return self.__repr__()
+        s = super(FloatWithUnit, self).__str__()
+        return "{} {}".format(s, self._unit)
 
     def __add__(self, other):
         if not hasattr(other, "unit_type"):
@@ -389,6 +390,20 @@ class FloatWithUnit(float):
                              unit_type=self._unit_type,
                              unit=self._unit)
 
+    def __getnewargs__(self):
+        """Function used by pickle to recreate object."""
+        return float(self), self._unit, self._unit_type
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state["val"] = float(self)
+        #print("in getstate %s" % state)
+        return state
+
+    def __setstate__(self, state):
+        #print("in setstate %s" % state)
+        self._unit = state["_unit"]
+
     @property
     def unit_type(self):
         return self._unit_type
@@ -477,6 +492,19 @@ class ArrayWithUnit(np.ndarray):
     def unit(self):
         return self._unit
 
+    def __reduce__(self):
+        #print("in reduce")
+        reduce = list(super(ArrayWithUnit, self).__reduce__())
+        #print("unit",self._unit)
+        #print(reduce[2])
+        reduce[2] = {"np_state": reduce[2], "_unit": self._unit}
+        return tuple(reduce)
+
+    def __setstate__(self, state):
+        #print("in setstate %s" % str(state))
+        super(ArrayWithUnit, self).__setstate__(state["np_state"])
+        self._unit = state["_unit"]
+
     def __repr__(self):
         return "{} {}".format(np.array(self).__repr__(), self.unit)
 
