@@ -32,6 +32,7 @@ import os
 import math
 import numpy as np
 import tempfile
+from pymatgen.core.lattice import Lattice
 from pymatgen.symmetry.finder import SymmetryFinder
 from pymatgen.electronic_structure.dos import Dos, Spin
 from pymatgen.electronic_structure.plotter import DosPlotter
@@ -151,12 +152,22 @@ class BoltztrapRunner():
         self._make_struc_file(os.path.join(path, "boltztrap.struct"))
         self._make_intrans_file(os.path.join(path, "boltztrap.intrans"))
 
-    def run(self, prev_sigma=None):
-        temp_dir = tempfile.mkdtemp()
+    def run(self, prev_sigma=None, path_dir=None):
         dir_bz_name = "boltztrap"
-        path_dir = os.path.join(temp_dir, dir_bz_name)
-        os.mkdir(path_dir)
+        path_dir_orig = path_dir
+        if path_dir is None:
+            temp_dir = tempfile.mkdtemp()
+            path_dir_orig = temp_dir
+            path_dir = os.path.join(temp_dir, dir_bz_name)
+        else:
+            path_dir = os.path.join(path_dir_orig, dir_bz_name)
+        if not os.path.exists(path_dir):
+            os.mkdir(path_dir)
+        else:
+            for c in os.listdir(path_dir):
+                os.remove(path_dir+"/"+c)
         os.chdir(path_dir)
+
         self._make_all_files(path_dir)
         if self._bs.is_spin_polarized:
             p = subprocess.Popen(["x_trans", "BoltzTraP", "-so"],
@@ -185,15 +196,37 @@ class BoltztrapRunner():
                 self._make_intrans_file(os.path.join(path_dir,
                                                      dir_bz_name + ".intrans"))
                 if self.lpfac > 100:
-                    raise BoltztrapError(
-                        "lpfac higher than 100 and still a warning")
-                self.run()
-
+                    raise BoltztrapError("lpfac higher than 100 and still a warning")
+                self.run(path_dir_orig)
+        #here we check if the doping levels were well computed
+        #sometimes boltztrap mess this up because of two small energy grids
+        analyzer = BoltztrapAnalyzer.from_files(path_dir)
+        doping_ok = True
+        for doping in ['n', 'p']:
+            for c in analyzer.mu_doping[doping]:
+                if len(analyzer.mu_doping[doping][c])!=len(analyzer.doping[doping]):
+                    doping_ok = False
+                    break
+                if doping == 'p' and \
+                        sorted(analyzer.mu_doping[doping][c], reverse=True) != analyzer.mu_doping[doping][c]:
+                    doping_ok = False
+                    break
+                if doping == 'n' and sorted(analyzer.mu_doping[doping][c]) != analyzer.mu_doping[doping][c]:
+                    doping_ok=False
+                    break
+        if not doping_ok:
+            self.energy_grid /= 10
+            print "lowers energy grid to "+str(self.energy_grid)
+            if self.energy_grid < 0.00005:
+                raise BoltztrapError("energy grid lower than 0.00005 and still no good doping")
+            self._make_intrans_file(path_dir + "/" + dir_bz_name + ".intrans")
+            self.run(prev_sigma=None, path_dir=path_dir_orig)
         analyzer = BoltztrapAnalyzer.from_files(path_dir)
         #here, we test if a property (eff_mass tensor) converges
         if prev_sigma is None or \
                 abs(sum(analyzer.get_eig_average_eff_mass_tensor()['n']) / 3
-                    - prev_sigma) / prev_sigma > 0.01:
+                        - prev_sigma)\
+                / prev_sigma > 0.05:
             if prev_sigma is not None:
                 print abs(sum(analyzer.get_eig_average_eff_mass_tensor()['n'])
                           / 3 - prev_sigma) / prev_sigma, \
@@ -201,12 +234,11 @@ class BoltztrapRunner():
                     analyzer.get_average_eff_mass_tensor(300, 1e18)
             self.lpfac *= 2
             if self.lpfac > 100:
-                raise BoltztrapError(
-                    "lpfac higher than 100 and still a warning")
+                raise BoltztrapError("lpfac higher than 100 and still not converged")
             self._make_intrans_file(path_dir + "/" + dir_bz_name + ".intrans")
             self.run(
                 prev_sigma=sum(analyzer.get_eig_average_eff_mass_tensor()
-                               ['n']) / 3)
+                               ['n']) / 3, path_dir=path_dir_orig)
         else:
             print "converged", \
                 abs(sum(analyzer.get_eig_average_eff_mass_tensor()['n'])/3
@@ -235,10 +267,10 @@ class BoltztrapAnalyzer():
 
     def __init__(self, gap, mu_steps, cond, seebeck, kappa, hall, doping,
                  mu_doping, seebeck_doping, cond_doping, kappa_doping,
-                 hall_doping, dos, warning):
+                 hall_doping, dos, carrier_conc, vol, warning):
         """
         Constructor taking directly all the data generated by Boltztrap. You
-        want probably use it directly but instead use the from_files and
+        won't probably use it directly but instead use the from_files and
         from_dict methods.
 
         Args:
@@ -313,8 +345,13 @@ class BoltztrapAnalyzer():
                 levels in doping and each Hall tensor is represented by a 27
                 coefficients list.
                 The units are m^3/C
+            the concentration of carriers:
+                in electron (or hole) per unit cell
             dos:
-                The dos computed by Boltztrap given as a pymatgen Dos object
+                The dos computed by Boltztrap
+                given as a pymatgen Dos object
+            vol:
+                the volume of the unit cell in angstrom cube (A^3)
             warning:
                 True if Boltztrap spitted out a warning
         """
@@ -332,12 +369,14 @@ class BoltztrapAnalyzer():
         self.cond_doping = cond_doping
         self.kappa_doping = kappa_doping
         self.hall_doping = hall_doping
+        self.carrier_conc = carrier_conc
         self.dos = dos
+        self.vol = vol
 
     @staticmethod
     def _make_boltztrap_analyzer_from_data(
             data_full, data_hall, data_dos, temperature_steps, mu_steps,
-            efermi, gap, doping, data_doping_full, data_doping_hall,
+            efermi, gap, doping, data_doping_full, data_doping_hall, vol,
             warning=False):
         """
         Make a BoltztrapAnalyzer object from raw data typically parse from
@@ -347,10 +386,10 @@ class BoltztrapAnalyzer():
         seebeck = {t: [] for t in temperature_steps}
         kappa = {t: [] for t in temperature_steps}
         hall = {t: [] for t in temperature_steps}
+        carrier_conc = {t: [] for t in temperature_steps}
         dos_full = {'energy': [], 'density': []}
         warning = warning
         new_doping = {'p': [], 'n': []}
-        print doping
         for d in doping:
             if d > 0:
                 new_doping['p'].append(d)
@@ -367,8 +406,8 @@ class BoltztrapAnalyzer():
                         'n': {t: [] for t in temperature_steps}}
         hall_doping = {'p': {t: [] for t in temperature_steps},
                        'n': {t: [] for t in temperature_steps}}
-
         for d in data_full:
+            carrier_conc[d[1]].append(d[2])
             tens_cond = [[d[3], d[4], d[5]],
                          [d[6], d[7], d[8]],
                          [d[9], d[10], d[11]]]
@@ -420,12 +459,10 @@ class BoltztrapAnalyzer():
             dos_full['density'].append(t[1])
 
         dos = Dos(efermi, dos_full['energy'], {Spin.up: dos_full['density']})
-        print new_doping
-        print mu_doping
         return BoltztrapAnalyzer(
             gap, mu_steps, cond, seebeck, kappa, hall, new_doping, mu_doping,
-            seebeck_doping, cond_doping, kappa_doping, hall_doping, dos,
-            warning)
+            seebeck_doping, cond_doping, kappa_doping, hall_doping, dos, carrier_conc,
+            vol, warning)
 
     def get_mu_bounds(self, temp=300):
         return min(self.mu_doping['p'][temp]), max(self.mu_doping['n'][temp])
@@ -569,13 +606,18 @@ class BoltztrapAnalyzer():
             with open(os.path.join(path_dir, "fort.27"), 'r') as f:
                 for line in f:
                     if not line.startswith("#") and len(line) > 2:
-                        data_doping_hall.append([float(c)
-                                                 for c in line.split()])
+
+                        data_doping_hall.append([float(c) for c in line.split()])
+
+        with open(os.path.join(path_dir, "boltztrap.struct"), 'r') as f:
+            tokens = f.readlines()
+            vol = Lattice([[Length(float(tokens[i].split()[j]), "bohr").to("ang")
+                            for j in range(3)] for i in range(1, 4)]).volume
         return BoltztrapAnalyzer._make_boltztrap_analyzer_from_data(
             data_full, data_hall, data_dos, sorted([t for t in t_steps]),
-            sorted([Energy(m, "Ry").to("eV") for m in m_steps]), efermi,
-            Energy(gap, "Ry").to("eV"), doping, data_doping_full,
-            data_doping_hall, warning)
+            sorted([Energy(m, "Ry").to("eV") for m in m_steps]), efermi, Energy(gap, "Ry").to("eV"),
+            doping, data_doping_full, data_doping_hall, vol, warning)
+
 
     @property
     def to_dict(self):
@@ -592,7 +634,9 @@ class BoltztrapAnalyzer():
                    'cond_doping': self.cond_doping,
                    'kappa_doping': self.kappa_doping,
                    'hall_doping': self.hall_doping,
-                   'dos': self.dos.to_dict}
+                   'dos': self.dos.to_dict,
+                   'carrier_conc': self.carrier_conc,
+                   'vol': self.vol}
         return clean_json(results)
 
     @staticmethod
@@ -647,7 +691,8 @@ class BoltztrapAnalyzer():
              'n': {int(d): [_make_float_hall(v)
                             for v in data['hall_doping']['n'][d]]
                    for d in data['hall_doping']['n']}},
-            Dos.from_dict(data['dos']), str(data['warning']))
+            Dos.from_dict(data['dos']), data['carrier_conc'],
+            data['vol'], str(data['warning']))
 
 
 class BoltztrapPlotter():
@@ -709,6 +754,7 @@ class BoltztrapPlotter():
         plt.show()
 
     def plot_power_factor(self, temp=300):
+        print "blah"
         matrix = []
         for i in range(len(self._bz.mu_steps)):
             matrix.append(np.dot(np.dot(self._bz.seebeck[temp][i],
@@ -746,3 +792,18 @@ class BoltztrapPlotter():
         plotter = DosPlotter(sigma=0.05)
         plotter.add_dos("t", self._bz.dos)
         plotter.get_plot().show()
+
+    def plot_carriers(self, temp=300):
+        plt.semilogy(self._bz.mu_steps,
+                       abs(self._bz.carrier_conc[temp]/(self._bz.vol*1e-24)),
+                       linewidth=3.0, color='r')
+        self._plot_BG_limits()
+        self._plot_doping(temp)
+        plt.xlim(-0.5, self._bz.gap+0.5)
+        plt.ylim(1e14,1e22)
+        plt.ylabel("carrier concentration (cm-3)", fontsize=30.0)
+        plt.xlabel("E-E$_f$ (eV)", fontsize=30)
+        plt.xticks(fontsize=25)
+        plt.yticks(fontsize=25)
+        plt.show()
+
