@@ -5,6 +5,7 @@ Wrapper classes for Cif input and output from Structures.
 """
 
 from __future__ import division
+from pymatgen.symmetry.finder import SymmetryFinder
 
 __author__ = "Shyue Ping Ong"
 __copyright__ = "Copyright 2011, The Materials Project"
@@ -162,7 +163,7 @@ class CifParser(object):
 
         struct = Structure(lattice, allspecies, allcoords)
         if primitive:
-            struct = struct.get_primitive_structure()
+            struct = struct.get_primitive_structure().get_reduced_structure()
         return struct.get_sorted_structure()
 
     def get_structures(self, primitive=True):
@@ -201,7 +202,7 @@ class CifWriter:
     A wrapper around PyCifRW to write CIF files from pymatgen structures.
     """
 
-    def __init__(self, struct):
+    def __init__(self, struct, find_spacegroup=False):
         """
         Args:
             struct:
@@ -211,13 +212,17 @@ class CifWriter:
         latt = struct.lattice
         comp = struct.composition
         no_oxi_comp = Composition(comp.formula)
-        block["_symmetry_space_group_name_H-M"] = "P 1"
+        spacegroup = ("P 1", 1)
+        if find_spacegroup:
+            sf = SymmetryFinder(struct, 0.001)
+            spacegroup = (sf.get_spacegroup_symbol(), sf.get_spacegroup_number())
+        block["_symmetry_space_group_name_H-M"] = spacegroup[0]
         for cell_attr in ['a', 'b', 'c']:
             block["_cell_length_" + cell_attr] = str(getattr(latt, cell_attr))
         for cell_attr in ['alpha', 'beta', 'gamma']:
             block["_cell_angle_" + cell_attr] = float(getattr(latt, cell_attr))
         block["_chemical_name_systematic"] = "Generated by pymatgen"
-        block["_symmetry_Int_Tables_number"] = 1
+        block["_symmetry_Int_Tables_number"] = spacegroup[1]
         block["_chemical_formula_structural"] = str(no_oxi_comp
                                                     .reduced_formula)
         block["_chemical_formula_sum"] = str(no_oxi_comp.formula)
@@ -316,15 +321,20 @@ def _clean_cif(s):
     """
     clean = []
     lines = s.split("\n")
+    skip = False
     while len(lines) > 0:
         l = lines.pop(0)
+        if skip:
+            if l.strip().startswith("_") or l.strip() == "loop_":
+                skip = False
+            else:
+                continue
+
         if l.strip().startswith("_cgraph"):
-            l = lines.pop(0)
-            while not (l.strip().startswith("_") or l.strip() == "loop_"):
-                l = lines.pop(0)
-            continue
+            skip = True
         elif not l.strip().startswith("_eof"):
             clean.append(remove_non_ascii(l))
+
     return "\n".join(clean)
 
 
